import 'package:flutter/material.dart';

/// Tipos de animaciones
enum AnimationType { normal, fadeIn }

/// Main class, [context] is BuildContext of the app at this moment
/// [child] is the destination widget of the navigation,
/// [animation] is the animation type
class RouteTransitions {
  final BuildContext context;
  final Widget child;
  final bool replacement;
  final AnimationType animation;
  final Duration duration;

  RouteTransitions({
    @required this.context,
    @required this.child,
    this.replacement = false,
    this.animation = AnimationType.normal,
    this.duration = const Duration(milliseconds: 300),
  }) {
    switch (this.animation) {
      case AnimationType.normal:
        this._normalTransition();
        break;
      case AnimationType.fadeIn:
        this._fadeInTransition();
        break;
    }
  }

  /// Normal push page
  void _pushPage(Route route) => Navigator.push(context, route);

  /// Push page with replacement enabled
  void _pushReplacementPage(Route route) =>
      Navigator.pushReplacement(context, route);

  /// Normal transition
  void _normalTransition() {
    final route =
        MaterialPageRoute(builder: (BuildContext context) => this.child);
    this.replacement ? this._pushPage(route) : this._pushReplacementPage(route);
  }

  /// Fadein Transition controller
  void _fadeInTransition() {
    final route = PageRouteBuilder(
      pageBuilder: (_, __, ___) => this.child,
      transitionDuration: this.duration,
      transitionsBuilder: (_, animation, __, child) {
        return FadeTransition(
          child: child,
          opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
              CurvedAnimation(parent: animation, curve: Curves.easeOut)),
        );
      },
    );

    this.replacement ? this._pushReplacementPage(route) : this._pushPage(route);
  }
}
